      PROGRAM FSPLIT
C     Split file with many Fortran routines into separate files.
C     This is because MS-DOS compilers do not make separate 
C     modules in the object file.
C     Language: ANSI Standard Fortran-77
C     Written by Robert Thrun, March 1992 for Microsoft Fortran
C        modified for ANSI Standard Fortran, November 1995
C     Not copyrighted or restricted in any way.

      PARAMETER (MAXLIN=2000)

      CHARACTER*1 TAB
      CHARACTER*40 FNAME, LIBNAM

C     Buffer to hold leading comments
C     If an array on an Intel processor is larger than 128K, the element
C     size must divide evenly into 64K so that an element is not split 
C     on a segment boundary.  Isn't segmented memory wonderful?
      CHARACTER*128 LINE(MAXLIN)

C     Work images of input lines
      CHARACTER*72 IN1, INP

C     File exists
      LOGICAL FEXIST
C     An output file is currently open
      LOGICAL OUT

C     Can't use data statement for non-printing character.
      TAB = CHAR(9)

C     Standard Fortran can't get command line arguments.
   10 WRITE (*,*) 'Name of Fortran file to be split? '
      READ (*, '(A)') FNAME
      IF (FNAME .EQ. ' ')  STOP ' Blank file name'
      INQUIRE (FILE=FNAME, EXIST=FEXIST)
C     Assume name is misspelled
      IF (.NOT. FEXIST) GO TO 10
C     Can all compilers open filenames with leading blanks?
      OPEN (1, FILE=FNAME)

C     Make a response file for LIB.EXE to assemble a library
      OPEN (3, FILE='LIB.CMD')
      I = INDEX (FNAME, '.')
      L = LENTRM (FNAME)
C     Provide the name of the object library
      IF (I .EQ. 0) THEN
         LIBNAM = FNAME(1:L) // '.LIB'
      ELSE
         LIBNAM = FNAME(1:I) // 'LIB'
      ENDIF
      WRITE (3, '(A / ''Y  & '' )') LIBNAM

C     Number of component files
      NFILES = 0
C     Number of lines read from input
      NIN = 0
C     Number of lines in buffer
   40 N = 0
C     No routine name found yet
      OUT = .FALSE.
C     Number of lines in output file
      NOUT = 0
   50 N = N + 1
      IF (N .GT. MAXLIN) THEN
         WRITE (*, '('' More than'', I5, '' leading comments'')') MAXLIN
         STOP
      ENDIF
C     We read the lines and save them because it is possible to
C     have comment lines before we find out the routine name.
      READ (1, '(A)', END=250) LINE(N)
      NIN = NIN + 1

C     Copy to work variable, expand tab in cols. 1-6, 
C     skip comments, skip continuation lines, remove trailing comments,
C     squeeze out blanks, and convert to uppercase.
      I = 0
   60 I = I + 1
      IF (LINE(N)(I:I) .EQ. TAB) GO TO 65
      IN1(I:I) = LINE(N)(I:I)
      IF (I .LT. 6) GO TO 60
      GO TO 70
   65 IN1(I:6) = '      '
C     IN1 now has tab expanded
   70 IN1(7:72) = LINE(N)(I+1:72)

C     We are interested in statements that indicate the beginning and
C     end of Fortran routines.  We don't need to examine comments,
C     continuation lines, or blank lines.  
      IF (IN1(1:1) .EQ. 'C') GO TO 80
      IF (IN1(1:1) .EQ. 'c') GO TO 80
      IF (IN1(1:1) .EQ. '*') GO TO 80
C     Non-ANSI comment or metacommand
      IF (IN1(1:1) .EQ. '$') GO TO 80
C     ! comment
      IF (INDEX(IN1(1:6),'!') .NE. 0) GO TO 80
C     Blank line
      IF (IN1(7:7) .EQ. ' ') GO TO 80
C     Not continuation, may be valid
      IF (IN1(6:6) .EQ. ' ') GO TO 90
C     Not continuation, may be valid
      IF (IN1(6:6) .EQ. '0') GO TO 90
C     All we have left are continuation lines and illegal statements.

C     We get here if we have a comment, blank line, continuation line,
C     or certain forms of illegal Fortran statements.
C     Keep collecting lines in buffer if no output file.
   80 IF (.NOT. OUT) GO TO 50

C     An output file is currently open. Write to it.
C     Remove trailing blanks
      NC = LENTRM (LINE(1))
      IF (NC .EQ. 0) THEN
         WRITE (2, '(A)')
      ELSE
         WRITE (2, '(A)') LINE(1)(1:NC)
      ENDIF
C     Increment the output file line count
      NOUT = NOUT + 1
C     Buffer of leading comments is empty
      N = 0
C     Process next line of input
      GO TO 50

C     We get here if we have what looks like a valid statement, 
C     including Fortran routine declarations, data declarations,
C     or executable statements.
C     Cols 1:6 not used in this compressed line image
   90 INP(1:6) = '      '
      I1 = 6
      I2 = 6
C     Remove embedded blanks after col. 6
   95 I1 = I1 + 1
      IF (I1 .GT. 72) GO TO 100
      IF (IN1(I1:I1) .EQ. ' ') GO TO 95
      I2 = I2 + 1
      INP(I2:I2) = IN1(I1:I1)
      GO TO 95
C     Add trailing blanks
  100 IF (I2 .LT. 72) INP(I2+1:72) = '   '

C     We have INP with embedded blanks removed.  Convert to uppercase.
      DO 110 I = 6,I2
      IF (INP(I:I) .LT. 'a') GO TO 110
      IF (INP(I:I) .GT. 'z') GO TO 110
      INP(I:I) = CHAR(ICHAR(INP(I:I))-32)
  110 CONTINUE

C     Do we have an output file name?
C     If so, write raw line image, keep looking for END
      IF (OUT) GO TO 200

C     Look for PROGRAM, SUBROUTINE, FUNCTION, OR BLOCK DATA name
C     so that we can give a name to the output file.

C     Is this too short to be a keyword?
      IF (I2 .LT. 14) THEN
C        Unnamed main program
         FNAME = 'MAIN'

C     We could have something like PROGRAM=1.0, which is a main program.
      ELSEIF (INDEX(INP(14:I2),'=') .NE. 0) THEN
         FNAME = 'MAIN'

      ELSEIF (INP(7:13) .EQ. 'PROGRAM') THEN
C        CDC programs have arguments in the PROGRAM statement
         I2 = INDEX(INP(14:21),'(')
         IF (I2 .EQ. 0) THEN
C           Max 8 char in program name, MS-DOS file system limit
            FNAME = INP(14:21)
         ELSE
            FNAME = INP(14:MIN(21,I2+12))
         ENDIF
C        Call it MAIN if PROGRAM statement doesn't have a name.
         IF (FNAME .EQ. '        ') FNAME = 'MAIN'

      ELSEIF (INP(7:16) .EQ. 'SUBROUTINE') THEN
C        Subroutines do not always have arguments
         I2 = INDEX(INP(17:24),'(')
         IF (I2 .EQ. 0) THEN
C           Max 8 char in subroutine name
            FNAME = INP(17:24)
         ELSE
            FNAME = INP(17:MIN(24,I2+15))
         ENDIF

      ELSEIF (INP(7:15) .EQ. 'BLOCKDATA') THEN
C        Some compilers allow BLOCK DATA to have a name.
         IF (INP(16:23) .NE. '        ') THEN
C           Max 8 char in name
            FNAME = INP(16:23)
         ELSE
            FNAME = 'BLOCKDAT'
         ENDIF

C     There are 19 kinds of Functions!!!
C     Maybe this should be table-driven; less code, more computation.
C     This program could make an error if the first statement of
C     an unnamed main program declares an array named 'FUNCTION'
C     FNAME always has a max of 8 chars, plus extension.

      ELSEIF (INP(7:14) .EQ. 'FUNCTION') THEN
         FNAME = INP(15:MIN(22,INDEX(INP(15:22),'(')+13))
      ELSEIF (INP(7:18) .EQ. 'BYTEFUNCTION') THEN
         FNAME = INP(19:MIN(26,INDEX(INP(19:26),'(')+17))


      ELSEIF (INP(7:21) .EQ. 'INTEGERFUNCTION') THEN
         FNAME = INP(22:MIN(29,INDEX(INP(22:29),'(')+20))
      ELSEIF (INP(7:23) .EQ. 'INTEGER*1FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'INTEGER*2FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'INTEGER*4FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))

      ELSEIF (INP(7:21) .EQ. 'LOGICALFUNCTION') THEN
         FNAME = INP(22:MIN(29,INDEX(INP(22:29),'(')+20))
      ELSEIF (INP(7:23) .EQ. 'LOGICAL*1FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'LOGICAL*2FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:23) .EQ. 'LOGICAL*4FUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))

      ELSEIF (INP(7:18) .EQ. 'REALFUNCTION') THEN
         FNAME = INP(19:MIN(26,INDEX(INP(19:26),'(')+17))
      ELSEIF (INP(7:20) .EQ. 'REAL*4FUNCTION') THEN
         FNAME = INP(21:MIN(28,INDEX(INP(21:28),'(')+19))
      ELSEIF (INP(7:20) .EQ. 'REAL*8FUNCTION') THEN
         FNAME = INP(21:MIN(28,INDEX(INP(21:28),'(')+19))

      ELSEIF (INP(7:29) .EQ. 'DOUBLEPRECISIONFUNCTION') THEN
         FNAME = INP(30:MIN(37,INDEX(INP(30:37),'(')+28))

      ELSEIF (INP(7:21) .EQ. 'COMPLEXFUNCTION') THEN
         FNAME = INP(22:MIN(29,INDEX(INP(22:29),'(')+20))
      ELSEIF (INP(7:27) .EQ. 'DOUBLECOMPLEXFUNCTION') THEN
         FNAME = INP(28:MIN(23,INDEX(INP(28:35),'(')+26))
      ELSEIF (INP(7:24) .EQ. 'COMPLEX*16FUNCTION') THEN
         FNAME = INP(25:MIN(32,INDEX(INP(25:32),'(')+23))

      ELSEIF (INP(7:23) .EQ. 'CHARACTERFUNCTION') THEN
         FNAME = INP(24:MIN(31,INDEX(INP(24:31),'(')+22))
      ELSEIF (INP(7:16) .EQ. 'CHARACTER*') THEN
         I1 = INDEX (INP(17:29),'FUNCTION') 
         IF (I1 .EQ. 0) THEN
C           Unnamed main with CHARACTER variable
            FNAME = 'MAIN'
         ELSE
C           Name is after 'FUNCTION'
            I1 = I1 + 24
            FNAME = INP(I1:MIN(I1+7,INDEX(INP(I1:I1+7),'(')+I1-2))
         ENDIF

      ELSE
C        Unnamed main program
         FNAME = 'MAIN'
      ENDIF
C     Finally! We are done identifying the routine type.

C     Find an acceptable filename, write out the backlog of leading
C     comments (if any), and write out the current declaration line.

      NF = LENTRM (FNAME)
      FNAME(NF+1:NF+4) = '.FOR'
      INQUIRE (FILE=FNAME, EXIST=FEXIST)
      IF (.NOT. FEXIST) GO TO 160
      DO 150 I = 1,999
C     The I3.3 format writes leading zeroes
      WRITE (FNAME(NF+2:NF+4), '(I3.3)') I
      INQUIRE (FILE=FNAME, EXIST=FEXIST)
      IF (.NOT. FEXIST) GO TO 160
  150 CONTINUE

C     If the problem is more than 1000 routines that have names that
C     are identical in the first 8 characters, we could just make up
C     arbitrary names.
      WRITE (*,*)' Too many conflicting filenames: ',FNAME(1:NF+1),'*'
      STOP

  160 OPEN (2, FILE=FNAME)
C     We know routine name, output file  has been opened
      OUT = .TRUE.
      NFILES = NFILES + 1
C     Write backlog of leading comments and current line.
      DO 170 I = 1,N
      NC = LENTRM (LINE(I))
      IF (NC .EQ. 0) THEN
         WRITE (2, '(A)')
      ELSE
         WRITE (2, '(A)') LINE(I)(1:NC)
      ENDIF
  170 CONTINUE
C     Count of lines in current routine
      NOUT = NOUT + N
C     Leading comments yet to be written out
      N = 0

C     Read next line of big Fortran source file.
      GO TO 50

C     We get here if we know the routine name and already 
C     have an output file and the first 6 columns are blank.
C     Remove trailing blanks, write out the line,
C     then see if it was an END statement.
  200 NC = LENTRM (LINE(1))
      IF (NC .EQ. 0) THEN
         WRITE (2, '(A)')
      ELSE
         WRITE (2, '(A)') LINE(1)(1:NC)
      ENDIF
C     Count of lines written out.
      NOUT = NOUT + 1
C     No lines left in leading comment buffer
      N = 0
      IF (INP(7:9) .NE. 'END') GO TO 50

C     Process END statement.
C     Make sure this is not other valid statement starting with 'END'
C     Maybe we should test for ! comment on same line as valid END.
      IF (INP(10:72) .NE. '    ') GO TO 50
      CLOSE (2)
C     Tell which file was just written.
      WRITE (*, '(I6,''. '', A12,  I7, '' lines'')')
     +        NFILES, FNAME(1:12), NOUT

C     Add to response file for Microsoft LIB.EXE.
      FNAME(NF+1:NF+4) = '.OBJ'
C     The last line of LIB.CMD will have an unwanted '&' that must be
C     edited by hand.  Such are the problems of portable Fortran.
      WRITE (3, '(''+'', A, '' &'')') FNAME(1:12)

C     Read next routine
      GO TO 40

C     Clean up when we hit end of input file.
  250 IF (OUT) THEN
        DO 260 I = 1,N
           NC = LENTRM (LINE(I))
           IF (NC .EQ. 0) THEN
              WRITE (2, '(A)')
           ELSE
C             Flush the buffer, write out characters in last line
              WRITE (2, '(A)') LINE(I)(1:NC)
           ENDIF
  260   CONTINUE
        NOUT = NOUT + N
        WRITE (*, '(I7, '' lines'')') NOUT
      ENDIF
      WRITE (*, '(I8, '' lines read'', I6, '' routines'')') NIN, NFILES
      WRITE (3, '(//)')
      STOP  '  End of file'
      END

      INTEGER FUNCTION LENTRM (STRING)
C     This function is not needed for Microsoft Fortran 5.0
C     This is an emulation of a Fortran-90 and MS-Fortran function.
      CHARACTER*(*) STRING
      L = LEN (STRING)
C     Find last non-blank, starting from the right end of the string.
      DO 10 I = L,1,-1
         IF (STRING(I:I) .NE. ' ') THEN
            LENTRM = I
            RETURN
         ENDIF
   10 CONTINUE
      LENTRM = 0
      RETURN
      END
